@@ -58,12 +58,7 @@ class CollectRawPlugin(CollectorPlugin):
         self.get_logger().info(
             f"Downloading from {source_url + file_name} to file {target_dir + file_name}"
         )
-        download_creds = None
-        if creds is not None:
-            download_creds = (creds.username, creds.password)
 
-        response = download_file(
-            source_url + file_name, target_dir + file_name, download_creds
-        )
+        response = download_file(source_url + file_name, target_dir + file_name, creds)
 
         return Result.from_http_response(response)
